#==============================================================================
#
#  $Id$
#
"""
   The yed library.
   
   At present, this is just the application-layer stuff, but even so it 
   doesn't belong here and needs to be refactored out into other modules.
"""
#
#  Copyright (C) 2008 Google Inc.
#
#   This file is part of SpugEdit.
#
#   SpugEdit is free software: you can redistribute it and/or modify it under
#   the terms of the GNU Lesser General Public License as published by the
#   Free Software Foundation, either version 3 of the License, or (at your
#   option) any later version.
#
#   SpugEdit is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with SpugEdit.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

from yed.core import BufferType, createAttrConfig, Font, RegionConfig, \
    setDefaultBufferType, Struct
from yed import log
from yed import createBufferFromFile, TextRules, \
    AlignToBottomCmd, \
    AlignToTopCmd, \
    BackSpaceCmd, \
    BlockDeleteCmd, \
    BlockInsertCmd, \
    CommandLineCommandFactory, \
    CopyCmd, \
    CutCmd, \
    GotoCmd, \
    KeyAliasCmdFactory, \
    MovementCommandFactory, \
    PasteCmd, \
    PipeCmd, \
    ReflowCmd, \
    ReplaceCmd, \
    ReturnCmd, \
    SearchCommandFactory, \
    SearchCmd, \
    TabCmd, \
    UntabCmd, \
    YedCmdHandler

_config = RegionConfig(
    rules = TextRules(),
    font = Font('fixed', 6),

    bindings = Struct(None, {
        '<Return>': ReturnCmd,
        '<BackSpace>': BackSpaceCmd,
        '<Tab>': TabCmd,
        '<Shift-Tab>': UntabCmd,
        '<Control-Tab>': UntabCmd,
        '<Control-h>': KeyAliasCmdFactory('<Left>'),
        '<Control-H>': KeyAliasCmdFactory('<Shift-Left>'),
        '<Control-j>': KeyAliasCmdFactory('<Down>'),
        '<Control-J>': KeyAliasCmdFactory('<Shift-Down>'),
        '<Control-k>': KeyAliasCmdFactory('<Up>'),
        '<Control-K>': KeyAliasCmdFactory('<Shift-Up>'),
        '<Control-l>': KeyAliasCmdFactory('<Right>'),
        '<Control-L>': KeyAliasCmdFactory('<Shift-Right>'),
        '<Control-b>': KeyAliasCmdFactory('<Control-Left>'),
        '<Control-B>': KeyAliasCmdFactory('<Control-Shift-Left>'),
        '<Control-w>': KeyAliasCmdFactory('<Control-Right>'),
        '<Control-W>': KeyAliasCmdFactory('<Control-Shift-Right>'),
        '<Control-A>': KeyAliasCmdFactory('<Shift-Home>'),
        '<Control-E>': KeyAliasCmdFactory('<Shift-End>'),
        '<Control-d>': CutCmd,
        '<Control-y>': CopyCmd,
        '<Control-p>': PasteCmd,
        '<Control-q>': ReflowCmd,
        '<Control-bar>': PipeCmd,
        '<F7>': BlockInsertCmd,
        '<Control-F7>': BlockDeleteCmd,
        '<Control-n>': MovementCommandFactory(GotoCmd, False),
        '<Control-N>': MovementCommandFactory(GotoCmd, True),
        '<Control-f>': SearchCommandFactory(SearchCmd, select = False, 
                                            forward = True, 
                                            caseSensitive = True
                                            ),
        '<Control-F>': SearchCommandFactory(SearchCmd, select = True, 
                                            forward = True, 
                                            caseSensitive = True
                                            ),
        '<Control-r>': SearchCommandFactory(SearchCmd, select = False, 
                                            forward = False, 
                                            caseSensitive = True
                                            ),
        '<Control-R>': SearchCommandFactory(SearchCmd, select = True, 
                                            forward = False, 
                                            caseSensitive = True
                                            ),
        '<Control-v>': ReplaceCmd,
        '<Control-Prior>': AlignToTopCmd,
        '<Control-Next>': AlignToBottomCmd,
        '<Escape>': CommandLineCommandFactory('Command', YedCmdHandler()),
    }),
    
    # default attributes.        
    attrs = Struct(foreground = 'cyan', 
                   background = 'black',
                   insertbackground = 'green', 
                   selectbackground = 'darkgreen', 
                   selectforeground = 'yellow',
                   ),
)

c = createAttrConfig
class DefaultBufferType(BufferType):
    """
        This is a holder for system-wide default settings.
    """

    name = 'default'
    config = _config

    regions = Struct(
        search_match = c(
            foreground = 'white',
            background = 'midnight blue'
        ),
        error = c(foreground = 'red'),
        symbol = c(foreground = 'yellow'),
        comment = c(foreground = 'green'),
        string = c(foreground = 'magenta'),
        preproc = c(foreground = 'white'),
    )


    def restore(self, top, name, attrs):
        "Implements @BufferType.restore()"
        
        # make sure that we at least have a filename
        if not attrs.has_key('filename'):
            log.error('Unable to restore buffer of type %s with no filename '
                       'attribute.'
                      )
            return

        buf = createBufferFromFile(top, attrs['filename'], 
                                   attrs.get('realname')
                                   )
        self._restoreAttrs(buf, name, attrs)
    
    def _restoreAttrs(self, buf, name, attrs):    
        # we want to load some of the attribute info from the state 
        # information and get some of it from the file XXX this sucks.
        overrides = {}
        for attr in ('filename', 'realname', 'mtime', 'mode'):
            if buf.attrs.has_key(attr):
                overrides[attr] = buf.attrs[attr]
        buf.attrs = attrs
        buf.attrs.update(overrides)
        buf.name = name
        
        # preserve the buffer type if there was one specified
        if attrs.has_key('type'):
            buf.attrs['type'] = self.name

setDefaultBufferType(DefaultBufferType())
