#==============================================================================
#
"""
    Help buffer.
"""
#
#   Copyright (C) 2009 Google, Inc.
#
#   This file is part of SpugEdit.
#
#   SpugEdit is free software: you can redistribute it and/or modify it under
#   the terms of the GNU Lesser General Public License as published by the
#   Free Software Foundation, either version 3 of the License, or (at your
#   option) any later version.
#
#   SpugEdit is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with SpugEdit.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

from yed.core import Command, registerBufferType
from yed.data.docs import data as helpDocument
from yed import topBindings
from default import DefaultBufferType

class HelpBufferType(DefaultBufferType):    
    name = 'help'

def showHelpBuffer(top):
     buffer = top.createBuffer('SpugEdit Help')
     HelpBufferType().initialize(buffer, top)
     buffer.insert('1.0', helpDocument)

class HelpBufferCmd(Command):
   """
      Go to the help buffer if it exists, create it if it does not.
   """
   
   def process(self, context):
      # look for the help buffer
      for buffer in context.top.buffers:
         if buffer.attrs.get('type') == 'help':
            context.top.focus(buffer)
            return 'break'
      else:
         # not found: create it.d
         showHelpBuffer(context.top)
         context.top.focus(buffer)

      return 'break'

topBindings['<F1>'] = HelpBufferCmd
registerBufferType(HelpBufferType())


