#==============================================================================
#
#  $Id$
#
"""
   Buffer type definitions for shell script buffers.
"""
#
#  Copyright (C) 2008 Google Inc.
#
#   This file is part of SpugEdit.
#
#   SpugEdit is free software: you can redistribute it and/or modify it under
#   the terms of the GNU Lesser General Public License as published by the
#   Free Software Foundation, either version 3 of the License, or (at your
#   option) any later version.
#
#   SpugEdit is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Lesser General Public License for more details.
#
#   You should have received a copy of the GNU Lesser General Public License
#   along with SpugEdit.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

import re
from yed import Parser, ShellCommentRules
from yed.core import RegionConfig, registerBufferType
from default import DefaultBufferType

class ShellParser(Parser):
   tokens = [
      ('symbol', re.compile(r'[\{\}\[\]\(\)!@%\^&\*\-\+=\|\\:;<>,\.\?\/\~]'),
       None
       ),
      ('string', re.compile(r'"'), re.compile(r'(\\.|[^\\"])*"')),
      ('string', re.compile(r"'"), re.compile(r"(\\.|[^\\'])*'")),
      ('comment', re.compile(r'#.*'), None),
      (None, re.compile(r'[^\{\}\[\]\(\)!@%\^&\*\-\+=\|\\:;<>,\.\?\/\~\'"#]+'),
       None
       ),
   ]

class ShellRules(ShellCommentRules):
   def getParserClass(self):
      return ShellParser

class ShellBufferType(DefaultBufferType):
    
    name = 'shell'
    extensions = ['.sh']
    firstLinePatterns = [
        re.compile(r'#!.*sh')
    ]
    config = RegionConfig(DefaultBufferType.config, rules = ShellRules())

registerBufferType(ShellBufferType())
